#include <signal.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

/* Ce programme montre comment bloquer et débloquer des signaux.

   Essayer "kill -USR1 <pid>" dans la ligne de commande avec le pid
   fourni par le programme.

   Certains appels système (p.ex. read, wait, ...) ou de libraire
   (p.ex. getchar qui renvoye vers read) terminent avec une erreur
   lorsqu'une interruption intervient. Par exemple, getchar() sera
   interrompu par une livraison de SIGUSR.

   Ce comportement est par ailleurs configurable par le drapeau
   SA_RESTART dans l'appel sigaction.

   Voir aussi signal (7), section "Interruption of system calls".
*/

void handler ()
{
	printf("coucou\n");
}

int main ()
{
	sigset_t newmask;
	struct sigaction action;
	memset(&action,0,sizeof(action));

	// décommenter la ligne suivant pour éviter que getchar()
	// soit interrompu par les signaux.
	// action.sa_flags = SA_RESTART;

	action.sa_handler = handler;
	sigaction(SIGUSR1,&action,NULL);

	printf("my pid is %d\n",getpid());

	sigemptyset(&newmask);
        sigaddset(&newmask,SIGUSR1);

	while (1)
	{
		char c;
		sigprocmask(SIG_BLOCK, &newmask, NULL);
		printf("SIGUSR1 now BLOCKED, press enter to unblock\n");
		c = getchar();
		printf("c = %d, errno = %d\n",c,errno);

		sigprocmask(SIG_UNBLOCK, &newmask, NULL);
		printf("SIGUSR1 now UNBLOCKED, press enter to block\n");

		// si getchar() est interrompu, il renvoie -1.
		c = getchar();
		printf("c = %d, errno = %d\n",c,errno);
	}
}
